/***************************************************************************/
/*                                                                         */
/* File: brain.cpp                        Date: 14-08-2004                 */
/*                                                                         */
/* Auth: Ben Kenwright                    Email: bkenwright@xbdev.net      */
/*                                                                         */
/* Project: Making a simple IRC Program.  A Bot that thinks and does more! */
/*                                                                         */
/***************************************************************************/ 
/*
   Our whole code has been built around a single class which encapsulates
   all the working code.
   I've called it Brain!  No particular reason, as its the first step to
   creating a smart bot...possibly adding script loading ability and other
   such things with time.

   We use ThreadProc(..) to lauch the main loop of our class, as it uses
   basic sockets...so we don't want to lockup our main app.  As we call
   DoWork(..) member function and it starts there and will keep checking
   for new messages etc and parsing them for what they mean.
*/
/***************************************************************************/ 

#pragma once

/***************************************************************************/ 


#include "brain.h"  // class definition
#include "xstr.h"   // xstrstr(...)
#include "xdata.h"  // stData structure


/***************************************************************************/ 

extern Brain g_brain; // in main.cpp
extern stData g_data; // in xdata.cpp


/***************************************************************************/ 
/*                                                                         */
/*  Name: ThreadProc(..)                          Date: 14-08-2004         */
/*                                                                         */
/*  Auth: bkenwright@xbdev.net                                             */
/*                                                                         */
/*  Description:  This method starts our Brain class working - it is       */
/*                passed a pointer to the class when its called.           */
/*                It is called as a new 'thread' with _beginthread so our  */
/*                socket code runs independent of our main thread.         */
/*                                                                         */
/*  Parameters:   Pass a pointer to the 'Brain' class so the function can  */
/*                launch the 'DoWork(..)' member function of Brain class   */
/*                                                                         */
/*  Returns:      void - nothing                                           */
/*                                                                         */
/***************************************************************************/ 

void ThreadProc(void *param)
{
	Brain * pbrain = static_cast<Brain*>(param);
	pbrain->DoWork();

}//End ThreadProc(..)



/***************************************************************************/ 
/*                                                                         */
/*  Name: Brain::Brain(..)                        Date: 14-08-2004         */
/*                                                                         */
/*  Auth: bkenwright@xbdev.net                                             */
/*                                                                         */
/*  Description:  Sets up our class, we use rand() a few times, so we      */
/*     initialize its starting value here.  We also set the which data     */
/*     we'll use.  So we set our pointer.                                  */
/*     Also, we keep track of the connection!  If its connected we we set  */
/*     bFirstTime to true.                                                 */
/*                                                                         */
/*  Parameters:   non                                                      */
/*                                                                         */
/*  Returns:      void - nothing                                           */
/*                                                                         */
/***************************************************************************/ 
Brain::Brain()
{
	srand( (unsigned int)GetTickCount() );
	m_pdata = &g_data;
	bFirstTime = true;
}//End Brain(..)



/***************************************************************************/ 
/*                                                                         */
/*  Name: Brain::RandomSayings()                  Date: 14-08-2004         */
/*                                                                         */
/*  Auth: bkenwright@xbdev.net                                             */
/*                                                                         */
/*  Description:  Each time we call this function - it will choose from    */
/*     one of the sayings in our list of funny sayings in the data         */
/*     structure in xdata.h and xdata.cpp                                  */
/*                                                                         */
/*  Parameters:   non                                                      */
/*                                                                         */
/*  Returns:      void - nothing                                           */
/*                                                                         */
/***************************************************************************/ 
void Brain::RandomSayings()
{
	// Get a random number
    int r = rand();

	int iNumSayings  = m_pdata->inumsayings;
	stSayings * pSay = m_pdata->saying;
	
	// Make our random number between 0 and the number of sayings we have
	r = r % iNumSayings;  // 0->iNumSayings-1

	char *szmessage = pSay[r].str;
	int  type       = pSay[r].type;

	// Choose if the saying is a normal text out or if we want it colour
	// coded to show its a thinking voice :)
	if( type == SPEAK )
	{
		SendChannel(szmessage);
	}
	else
	{
		SendMeChannel(szmessage);
	}//End if else(..)

}// End RandomSayings(..)


/***************************************************************************/ 
/*                                                                         */
/*  Name: Brain::JoinChannel(..)                  Date: 14-08-2004         */
/*                                                                         */
/*  Auth: bkenwright@xbdev.net                                             */
/*                                                                         */
/*  Description:  We call this once our connection to the server has been  */
/*     setup.  We pass the name of the channel we want to join...and thats */
/*     it...simle eh?                                                      */
/*                                                                         */
/*  Parameters:   String which is the name of the channel we want to join  */
/*     e.g. "xbdev"                                                        */
/*                                                                         */
/*  Returns:      void - nothing                                           */
/*                                                                         */
/***************************************************************************/ 
void Brain::JoinChannel(char * str)
{
	dbg("JoinChannel(..)\n");
    sprintf(aa, "JOIN #%s\r\n", str ); // Join a chat channel
	dbg(aa);
	send(m_socket, aa, (int)strlen(aa), 0);
}// End JoinChannel(..)


/***************************************************************************/ 
/*                                                                         */
/*  Name: Brain::SendNick()                       Date: 14-08-2004         */
/*                                                                         */
/*  Auth: bkenwright@xbdev.net                                             */
/*                                                                         */
/*  Description:  Whenever we connect to a server we need to say which     */
/*     nickname where going to use - everyone must have a unique nickname  */
/*     so we use this function to send our nickname :)                     */
/*                                                                         */
/*  Parameters:   none - we should pass the name of our nick, but we use   */
/*     the name of the nick from our stData structure in xdata.cpp         */
/*                                                                         */
/*  Returns:      void - nothing                                           */
/*                                                                         */
/***************************************************************************/ 
void Brain::SendNick()
{
	dbg("Sending NICK << etc \n");

	char * sznick = m_pdata->nick;

	sprintf(aa, "NICK %s\r\n"              // e.g. "NICK bobyvodi\r\n"
			    "USER %s %s %s :%s\r\n",   // e.g. "USER bobyvodi bobyvodi bobyvodi :bobyvodi\r\n"
				sznick,
				sznick,sznick,sznick,sznick);

    send(m_socket, aa, (int)strlen(aa), 0);
}


/***************************************************************************/ 
/*                                                                         */
/*  Name: Brain::SendChannel                      Date: 14-08-2004         */
/*                                                                         */
/*  Auth: bkenwright@xbdev.net                                             */
/*                                                                         */
/*  Description:  Chat chat chat...we want to send a message to the        */
/*     channel so we can speak to others!  Well this is our magic function */
/*     the definition of the text string is in xdata.h.  As we use         */
/*     STR_MSG to define our string.                                       */
/*                                                                         */
/*  Parameters:   szmessage - string we want to send to the channel        */
/*                szchannel - which channel we wnat to send the message to */
/*                   the default is the channel defined in xdata.cpp       */
/*                                                                         */
/*  Returns:      void - nothing                                           */
/*                                                                         */
/***************************************************************************/ 
void Brain::SendChannel(char *szmessage, char * szchannel /*= 0*/)
{
	// If the user hasn't specified a channel, then we'll just use our default
	// channel on connect.
	if( szchannel==0 ) szchannel = m_pdata->channel;

	EnterCriticalSection(&m_critSec);

	dbg("SendMeChannel(..)\n");
	sprintf(m_pShareBuf, STR_MSG, szchannel, szmessage);
	dbg(m_pShareBuf);
	send(m_socket, m_pShareBuf, (int)strlen(m_pShareBuf), 0);

	LeaveCriticalSection(&m_critSec);
}// SendChannel(..)


/***************************************************************************/ 
/*                                                                         */
/*  Name: Brain::SendMeChannel                    Date: 14-08-2004         */
/*                                                                         */
/*  Auth: bkenwright@xbdev.net                                             */
/*                                                                         */
/*  Description:  This is the same as doing '/me cool message'..in mirc    */
/*      well the /me part anyhow...we pass the message to the function     */
/*                                                                         */
/*  Parameters:   szmessage - string we want to send to the channel        */
/*                szchannel - which channel we wnat to send the message to */
/*                   the default is the channel defined in xdata.cpp       */
/*                                                                         */
/*  Returns:      void - nothing                                           */
/*                                                                         */
/***************************************************************************/ 
void Brain::SendMeChannel(char *szmessage, char * szchannel /*= 0*/ )
{
	// If the user hasn't specified a channel, then we'll just use our default
	// channel on connect.
	if( szchannel==0 ) szchannel = m_pdata->channel;

	EnterCriticalSection(&m_critSec);

	dbg("SendMeChannel(..)\n");
	sprintf(m_pShareBuf, STR_ME, szchannel, szmessage);
	dbg(m_pShareBuf);
	send(m_socket, m_pShareBuf, (int)strlen(m_pShareBuf), 0);

	LeaveCriticalSection(&m_critSec);
}// End SendMeChannel(..)



/***************************************************************************/ 
/*                                                                         */
/*  Name: Brain::Pong                             Date: 14-08-2004         */
/*                                                                         */
/*  Auth: bkenwright@xbdev.net                                             */
/*                                                                         */
/*  Description:  IRC is always pinging us to see if we've died or timed   */
/*      out - so if we don't pong it back with the time it sent us it will */
/*      just kick us off.                                                  */
/*                                                                         */
/*  Parameters:   str - A copy of the string which we received, which      */
/*      contains the ping and the value it was sent.                       */
/*                                                                         */
/*  Returns:      void - nothing                                           */
/*                                                                         */
/***************************************************************************/ 
void Brain::Pong(char * str)
{
	memset(bb, '\0', sizeof(bb) );
    strcpy(bb, "PONG :");
    strncat(bb, aa+6, 8);
	send(m_socket, bb, (int)strlen(bb), 0);
	dbg(bb);

	if( bFirstTime )
	{
		SendNick();
		bFirstTime = false;
	}
}//End Pong(..)

	
/***************************************************************************/ 
/*                                                                         */
/*  Name: SomeoneJoinChannel                      Date: 14-08-2004         */
/*                                                                         */
/*  Auth: bkenwright@xbdev.net                                             */
/*                                                                         */
/*  Description:  We wanted to respond to the channel everytime someone    */
/*      joins.  So if we detect 'JOIN' in the incomming string...it means  */
/*      someone has joined.  So we pass a pointer to the received string   */
/*      and extract the name of the person who's just joined.  Then we     */
/*      we send a simple message to acknoledge they've arrived :)          */
/*                                                                         */
/*  Parameters:   Pointer to the received string.                          */
/*                                                                         */
/*  Returns:      void - nothing                                           */
/*                                                                         */
/***************************************************************************/ 
void Brain::SomeoneJoinChannel(char * str )
{
	if( bFirstTime ) return;

	// First lets get the nick name
	memset(bb, '\0', sizeof(bb));
	// :kl0wn!~kl0wn@g00d.girls.like.to.t0uch.us JOIN :#XFactor
	int i=0;
	while( aa[i] != '~' )
	{
		bb[i] = aa[i+1];
		i++;
	}
	bb[i-2] = '\0';

	// Okay, bb is the nick name
		
	char buf[200]; // temp buffer
		
	// Test purposes, lets just hand them a cookie or say hello
	int either = rand()%2;
	if( either == 0 )
	{
		sprintf(buf, "gives %s a cookie ", bb );
		SendMeChannel(buf, m_pdata->channel);
	}
	else
	{
		sprintf(buf, "hey %s", bb );
		SendMeChannel(buf, m_pdata->channel);
	}
}// End SomeoneJoinChannel(..)


/***************************************************************************/ 
/*                                                                         */
/*  Name: Brain::ParseReceived                    Date: 14-08-2004         */
/*                                                                         */
/*  Auth: bkenwright@xbdev.net                                             */
/*                                                                         */
/*  Description:  You could say this is our sort of decision making part   */
/*     of our brain.  We receive strings from our server..and we have to   */
/*     determine what to do with them...they could be if someones entered  */
/*     a message..if the server has pinged us...if someone kicks us etc.   */
/*     So we respond to each event here...we could add more event handling */
/*     and things later on :)                                              */
/*                                                                         */
/*  Parameters:   Pointer to the received string.                          */
/*                                                                         */
/*  Returns:      void - nothing                                           */
/*                                                                         */
/***************************************************************************/ 
void Brain::ParseReceived(char * str )
{
	dbg(aa);

	if( xstrstr(aa, "JOIN :") ) SomeoneJoinChannel(aa);

	if( xstrstr(aa, "PING") ) Pong(aa);

	if( xstrstr(aa, "End of /MOTD command") )JoinChannel( m_pdata->channel );

	if( xstrstr(aa, "Cannot send to channel") ) JoinChannel( m_pdata->channel );

	if( xstrstr(aa, "KICK")  ) JoinChannel( m_pdata->channel );

}// ParseReceived(..)


/***************************************************************************/ 
/*                                                                         */
/*  Name: Brain::DoWork()(...)                    Date: 14-08-2004         */
/*                                                                         */
/*  Auth: bkenwright@xbdev.net                                             */
/*                                                                         */
/*  Description:  ;HEART' ...well that what this is...we start here        */
/*     and we init all our socket code etc...and then we constantly        */
/*     receive strings from our server...and we pass them along to the     */
/*     parser function above to determine what to do with them :)          */
/*                                                                         */
/*  Parameters:   void - nothing                                           */
/*                                                                         */
/*  Returns:      void - nothing                                           */
/*                                                                         */
/***************************************************************************/ 
void Brain::DoWork()
{	
    m_pbuf = new char[100];
    memset(m_pbuf, '\0', 90 );

	m_pShareBuf = new char[100];
        
	InitializeCriticalSection(&m_critSec);

    WSAStartup( 0x101, &m_ws);
		
    m_socket = socket(AF_INET, SOCK_STREAM, 0);

    a.sin_family = AF_INET;
	a.sin_port = htons( m_pdata->port ); // e.g. 6667

    hostent *ff = gethostbyname( m_pdata->szHostName ); // e.g. irc.efnet.nl
    a.sin_addr = *((LPIN_ADDR)*ff->h_addr_list);

    memset(aa, '\0', sizeof(aa));

	dbg("Connecting...");

    int d = connect(m_socket, (struct sockaddr *)&a, sizeof(a));
    recv(m_socket, aa, 5000, 0);
    dbg(aa);

	
	SendNick();


    double NowTime = GetTickCount();
    double OldTime = GetTickCount();

    dbg(" Thread is Running!\n");


	int m_tick = 0;

    int ii = 1;
    while( ii != 0)
    {
        memset(aa, '\0', sizeof(aa) );
        ii = recv(m_socket, aa, 5000, 0);

		if( ii == 0 ) memset(aa, '\0', sizeof(aa) );

        ParseReceived(aa);

		// What happens here, is each time we receive text...for some reason or another
		// either a ping or someone types something in the channel..we get here..and
		// we increment m_tick.
		// Now every 20 ticks we call 'RandomSaying' to say something in the channel
		// to show where still alive :)
		{
			m_tick++;

			// Display the tick value to our window
			printf("tick:%d   tick-mod-60: %d  \r\n", m_tick, m_tick%60);

			if( (m_tick % 20) == 0 )
				RandomSayings();
		}
				
    }//End while(..)

    printf("end thread\n");
    
    _endthread();

}//End DoWork(..)




