/***************************************************************************/
/*                                                                         */
/* File: brain.h                          Date: 14-08-2004                 */
/*                                                                         */
/* Auth: Ben Kenwright                    Email: bkenwright@xbdev.net      */
/*                                                                         */
/* Project: Making a simple IRC Program.  A Bot that thinks and does more! */
/*                                                                         */
/***************************************************************************/ 
/*
   Our whole code has been built around a single class which encapsulates
   all the working code.
   I've called it Brain!  No particular reason, as its the first step to
   creating a smart bot...possibly adding script loading ability and other
   such things with time.

   We use ThreadProc(..) to lauch the main loop of our class, as it uses
   basic sockets...so we don't want to lockup our main app.  As we call
   DoWork(..) member function and it starts there and will keep checking
   for new messages etc and parsing them for what they mean.
*/
/***************************************************************************/ 
/*
   References:
   http://www.thecodeproject.com/threads/crtmultithreading.asp
*/
/***************************************************************************/

#pragma once

 
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
#include <stdio.h>     // sprintf(..)

#include <process.h>   // _beginthread, _endthread
//#include <stddef.h>
#include <stdlib.h>


#include <winsock.h>   // WinSock(..) connect(..) send(..) 

#pragma comment(lib, "wsock32.lib")

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

#include "xdebug.h"   // dbg(..)
#include "xdata.h"
#include "xstr.h"     // xstrstr(..)


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////






/***************************************************************************/ 
/*                                                                         */
/*  Name: Brain class                             Date:14-08-2004          */
/*                                                                         */
/*  Auth: bkenwright@xbdev.net                                             */
/*                                                                         */
/*  Description:  This is our 'Main' class for the code!  It contains      */
/*  nearly all the code for what we want to do.  We create an instance of  */
/*  it, and then we use a second thread to do the work loop..which never   */
/*  returns...well only returns when the connection is closed.  The work   */
/*  loop is in member function DoWork(..)                                  */
/*                                                                         */
/*  GLOBALS!                                                               */
/*  The class uses a global set of data which is initialised on setup...   */
/*  the global data is called 'g_data' and is located in xdata.h and       */
/*  xdata.cpp                                                              */
/*                                                                         */
/***************************************************************************/ 

class Brain
{
	WSADATA m_ws;

	SOCKET m_socket;
    
	struct sockaddr_in a;
	
    char bb[5000];
	char aa[5000];

	char * m_pbuf;

	CRITICAL_SECTION	m_critSec;
	char *              m_pShareBuf;
	
	stData *            m_pdata;

	bool bFirstTime;

public:
	Brain();



    void RandomSayings();


	void JoinChannel(char * str);
	void SendNick();

	void SendChannel(char *szmessage, char * szchannel=0);

    void SendMeChannel(char *szmessage, char * szchannel = 0 );

	void Pong(char * str);

	void SomeoneJoinChannel(char * str );

	void ParseReceived(char * str );


	void DoWork();

};



/***************************************************************************/ 
/*                                                                         */
/*  Name: ThreadProc(..)                          Date:14-08-2004          */
/*                                                                         */
/*  Auth: bkenwright@xbdev.net                                             */
/*                                                                         */
/*  Description:  This method starts our Brain class working - it is       */
/*                passed a pointer to the class when its called.           */
/*                It is called as a new 'thread' with _beginthread so our  */
/*                socket code runs independent of our main thread.         */
/*                                                                         */
/*  Parameters:   Pass a pointer to the 'Brain' class so the function can  */
/*                launch the 'DoWork(..)' member function of Brain class   */
/*                                                                         */
/*  Returns:      void - nothing                                           */
/*                                                                         */
/***************************************************************************/ 
void ThreadProc(void *param);









